-- ml.lua
-- Some bits of the Magic Lantern scripting API 
-- To be executed right after loading LUA libraries, but before registering ML functions

-- make a list of LUA builtin functions
_LG = {}; 
for i in pairs(_G) do 
	if type(_G[i]) == 'function' then 
		_LG[i] = _G[i] 
	end 
end

function printf(fmt, ...)
	print(string.format(fmt, ...))
end

function dir(x) -- python smell in lua :)
	if type(x) == 'table' then
		local msg = ""
		local props = {}
		for i in pairs(x) do
			p = string.sub(i,1,5)
			if p == '_get_' or p == '_set_' then
				props[string.sub(i,6)] = 1;
			else
				msg = msg .. " " .. i
			end
		end
		if next(props) ~= nil then 
			msg = msg .. "; properties:"
			for i in pairs(props) do
				msg = msg .. " " .. i
			end
		end
		return msg
	else
		return tostring(x)
	end
end

_getset = {} -- properties

_getset.__index = function(table,key)
	local v
	v = rawget(table, key)
	if v ~= nil then return v end
	v = rawget(table, '_get_' .. tostring(key))
	if v ~= nil then return v() end
	v = rawget(table, '_get')
	if v ~= nil then return v(key) end
end

_getset.__newindex = function(table,key,value)
	local v
	v = rawget(table, '_set_' .. tostring(key))
	if v ~= nil then 
		v(value)
		return
	end

	v = rawget(table, '_set')
	if v ~= nil then
		return v(key,value) 
	end

	rawset(table,key,value)
end

if os then
	Q = function()
		os.exit()
	end
end
